import React from 'react';
import { bool, shape, string } from 'prop-types';
import { useScrollLock } from '@magento/peregrine/lib';

import { mergeClasses } from '@magento/venia-ui/lib/classify';
import Footer from '../Footer';
import Header from '../Header';
import defaultClasses from '@magento/venia-ui/lib/components/Main/main.css';

import customClasses from './main.css';
import FooterMarketing from "../FooterMarketing/footerMarketing";

import TopBanner from "../TopBanner/topBanner";

const Main = props => {
    const { children, isMasked } = props;
    const classes = mergeClasses(defaultClasses, props.classes);

    const rootClass = isMasked ? classes.root_masked : classes.root;
    const pageClass = isMasked ? classes.page_masked : classes.page;

    useScrollLock(isMasked);

    return (
        <main className={rootClass}>
            {/* Classes passed via props - Override a component's style​ Exercise */}
            <Header
                classes={{
                    open: customClasses.open,
                    closed: customClasses.closed
                }}
            />

            <TopBanner /> {/* Custom Component - Creating a new storefront component​ Exercise */}

            <div className={pageClass}>{children}</div>

            <FooterMarketing /> {/* Custom Component - Creating a new storefront component​ Topic */}

            {/* Classes passed via props - Override a component's style​ Topic */}
            <Footer
                classes={{
                    root: customClasses.customFooterRoot,
                    copyright: customClasses.customFooterCopyright
                }}
            />
        </main>
    );
};

export default Main;

Main.propTypes = {
    classes: shape({
        page: string,
        page_masked: string,
        root: string,
        root_masked: string
    }),
    isMasked: bool
};
